<!-- Generated by documentation.js. Update this documentation by updating the source code. -->

### Table of Contents

-   [Ctx](#ctx)
    -   [err](#err)
    -   [ppCalc](#ppcalc)
    -   [ppCalcAcc](#ppcalcacc)
-   [Buffer](#buffer)
-   [Beatmap](#beatmap)
    -   [applyMods](#applymods)
    -   [parse](#parse)
-   [DiffCalcCtx](#diffcalcctx)
    -   [DiffCalc](#diffcalc)

## Ctx

**`oppai.Ctx`**

Creates a pp calculation context for the current thread.
Each instance should not be used concurrently.

### err

Gets a description of the last error that occurred during oppai
calls. This should be checked after 'Beatmap.parse', 'Ctx.ppCalc',
'Ctx.ppCalcAcc' and 'DiffCalcCtx.diffCalc'.

**Examples**

```javascript
var ctx = oppai.Ctx();
var b = oppai.Beatmap(ctx);
var buf = oppai.Buffer(2000000);
b.parse("some_file.osu", buf, 2000000, true);
if (ctx.err()) {
    console.error(ctx.err());
    process.exit(1);
}
```

Returns **[string](https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/String)** description of the last error or undefined if none

### ppCalc

Calculates ppv2

**Parameters**

-   `aim` **[number](https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/Number)** aim difficulty (see 'DiffCalcCtx.diffCalc')
-   `speed` **[number](https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/Number)** speed difficulty (see 'DiffCalcCtx.diffCalc')
-   `b` **[Object](https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/Object)&lt;oppai.Beatmap>** the beatmap with map-modifying
                                       mods already applied
-   `usedMods` **uint32?** the mods bitmask
                                             (see 'Beatmap.applyMods') (optional, default `oppai.nomod`)
-   `combo` **uint16?** desired combo. 0xFFFF will assume full
                                     combo (optional, default `0xFFFF`)
-   `misses` **uint16?** amount of misses (optional, default `0`)
-   `c300` **uint16?** amount of 300s. 0xFFFF will automatically
                                    calculate this value based on the number
                                    of misses, 100s and 50s. (optional, default `0xFFFF`)
-   `c100` **uint16?** number of 100s (optional, default `0`)
-   `c50` **uint16?** number of 50s (optional, default `0`)
-   `scoreVersion` **uint32?** 1 or 2, affects accuracy pp (optional, default `1`)

**Examples**

```javascript
var ctx = oppai.Ctx();

var b = oppai.Beatmap(ctx);
var buf = oppai.Buffer(2000000);
b.parse("some_file.osu", buf, 2000000, true);

dctx = oppai.DiffCalcCtx(ctx);
diff = dctx.diffCalc(b);
res = ctx.ppCalc(diff.aim, diff.speed, b);
console.log(
    util.format(
        "\n%d aim\n%d speed\n%d acc\n%d pp\nfor %d%%",
        res.aimPp, res.speedPp, res.accPp, res.pp, res.accPercent
    )
);
```

Returns **[object](https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/Object)** an object containing 'accPercent', 'pp', 'accPp',
                  'speedPp' and 'aimPp'.

### ppCalcAcc

Same as 'Ctx.ppCalc' but uses accuracy percentage instead of number of
100/50s. Accuracy is automatically rounded to the closest 100/50 count.

**Parameters**

-   `aim` **[number](https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/Number)** aim difficulty (see 'DiffCalcCtx.diffCalc')
-   `speed` **[number](https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/Number)** speed difficulty (see 'DiffCalcCtx.diffCalc')
-   `b` **[Object](https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/Object)&lt;oppai.Beatmap>** the beatmap with map-modifying
                                       mods already applied
-   `accuracyPercent` **[number](https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/Number)?** the accuracy in percentage (optional, default `100.0`)
-   `usedMods` **uint32?** the mods bitmask
                                             (see 'Beatmap.applyMods') (optional, default `oppai.nomod`)
-   `combo` **uint16?** desired combo. 0xFFFF will assume full
                                     combo (optional, default `0xFFFF`)
-   `misses` **uint16?** amount of misses (optional, default `0`)
-   `scoreVersion` **uint32?** 1 or 2, affects accuracy pp (optional, default `1`)

**Examples**

```javascript
var ctx = oppai.Ctx();

var b = oppai.Beatmap(ctx);
var buf = oppai.Buffer(2000000);
b.parse("some_file.osu", buf, 2000000, true);

dctx = oppai.DiffCalcCtx(ctx);
diff = dctx.diffCalc(b);
res = ctx.ppCalc(diff.aim, diff.speed, b, 95.00);
console.log(
    util.format(
        "\n%d aim\n%d speed\n%d acc\n%d pp\nfor %d%%",
        res.aimPp, res.speedPp, res.accPp, res.pp, res.accPercent
    )
);
```

Returns **[object](https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/Object)** an object containing 'accPercent', 'pp', 'accPp',
                  'speedPp' and 'aimPp'.

## Buffer

**`oppai.Buffer`**

Allocates a new buffer to be used for beatmap parsing.
This should be at least as big as the .osu files you're going to
parse.

**Parameters**

-   `nbytes` **uint32** number of bytes to allocate

**Examples**

```javascript
buf = oppai.Buffer(2000000)
```

## Beatmap

**`oppai.Beatmap`**

Creates an empty Beatmap object. See 'Beatmap.parse' to fill this
object.

### applyMods

Applies map-changing mods. Note that this is currently not reversible
and you will have to re-parse the map to undo mods like DT.

**Parameters**

-   `usedMods` **uint32** the mods bitmask, which can be built by
                               bit-wise OR-ing the mod constants

**Examples**

```javascript
var ctx = oppai.Ctx();
var b = oppai.Beatmap(ctx);
var buf = oppai.Buffer(2000000);
b.parse("some_file.osu", buf, 2000000, true);
b.applyMods(oppai.hd | oppai.hr);
```

### parse

Parse .osu file into a 'oppai.Beatmap' object.

**Parameters**

-   `osuFile` **[string](https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/String)** path to the .osu file. If "-" is passed,
                              input will be read from stdin.
-   `disableCache` **[boolean](https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/Boolean)?**     by default, beatmaps are cached in a pre-parsed binary format
        to disk. Setting this to true disables this feature. (optional, default `false`)
-   `customCacheFolder` **[string](https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/String)?**     overrides the base cache directory. (optional, default `"path/to/current/executable"`)

**Examples**

```javascript
var ctx = oppai.Ctx();
var b = oppai.Beatmap(ctx);
var buf = oppai.Buffer(2000000);
b.parse("some_file.osu", buf, 2000000, true);
```

## DiffCalcCtx

**`oppai.DiffCalcCtx`**

Creates a difficulty calculation context for the current thread.
Each instance should not be used concurrently.

### DiffCalc

Calculates overall, aim and speed stars for a map.

**Parameters**

-   `b` **[object](https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/Object)&lt;oppai.Beatmap>** a parsed beatmap with map-changing
                                       mods already applied.
-   `withRhythmAwkwardness` **[boolean](https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/Boolean)?**     calculates and adds rhythm awkwardness to the returned object. (optional, default `false`)
-   `withNSingles` **[boolean](https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/Boolean)?**     calculates and adds the number of spacing singletaps (as seen by
        the difficulty calculator) to the returned object. These are
        based on spacing thresholds. (optional, default `false`)
-   `withNSinglesTiming` **[boolean](https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/Boolean)?**     calculates and adds the number of timing singletaps (1/2 notes)
        to the returned object. (optional, default `false`)
-   `withNSinglesThreshold` **[boolean](https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/Boolean)?**     calculates and adds the number of notes that are 1/2 or slower at
        singletapThreshold BPM. (optional, default `false`)
-   `singletapThreshold` **uint32?**     singletap threshold BPM for withNSinglesThreshold (optional, default `240`)

**Examples**

```javascript
var ctx = oppai.Ctx();

var b = oppai.Beatmap(ctx);
var buf = oppai.Buffer(2000000);
b.parse("some_file.osu", buf, 2000000, true);

dctx = oppai.DiffCalcCtx(ctx);
diff = dctx.diffCalc(b);
console.log(
    util.format(
        "\n%d stars\n%d aim stars\n%d speed stars",
        diff.stars, diff.aim, diff.speed
    )
);
```

Returns **[object](https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/Object)** an object containing 'stars', 'aim', 'speed' and,
    optionally, 'rhythmAwkwardness', 'nSingles', 'nSinglesTiming' and
    'nSinglesThreshold'.
